package page_votes

import (
	"code.electrolab.fr/it/vote.electrolab.fr/app/server"
	"code.electrolab.fr/it/vote.electrolab.fr/app/server/pages/common"
	"code.electrolab.fr/it/vote.electrolab.fr/model"
	"context"
	"fmt"
	"log"
	"net/http"
)

var (
	template = common.MustParseTemplate(TEMPLATE_STR)
)

func RegisterRoutes(mux *http.ServeMux, app *server.App) {
	mux.Handle("/events/Electrolab-AG-2020/votes", server.NewAppHandler(app, handleGetVotes))
}

func handleGetVotes(app *server.App, ctx context.Context, w http.ResponseWriter, r *http.Request) {
	var (
		err               error
		templateIndexVars = new(TemplateIndexVars)
	)

	templateIndexVars.PageTitle = "Vote Électrolab"

	templateIndexVars.User, err = model.FetchUser(app.Context, app.Service.DB, r.FormValue("private_token"))
	if err != nil {
		// Query has an error
		log.Printf("Query error: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	if templateIndexVars.User == nil {
		// Token is not valid
		w.WriteHeader(http.StatusForbidden)
		fmt.Fprintf(w, "Invalid token")
		return
	}

	// Token is valid, download the questions

	templateIndexVars.Questions, err = model.FetchQuestions(app.Context, app.Service.DB, templateIndexVars.User.Uuid)
	if err != nil {
		// Query has an error
		log.Printf("Query error 2: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	err = template.Execute(w, templateIndexVars)
	if err != nil {
		// Template has an error
		log.Printf("Template execution error: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

}
